<?php

class Thumb2xHandler
{
    /** @var modX $modx */
    public $modx;
    /** @var array Кэш для экземпляров Glide */
    protected $instances = [];

    /**
     *
     * @param modX $modx
     */
    public function __construct(modX &$modx)
    {
        $this->modx =& $modx;
    }

    /**
     * Получает или создает экземпляр Glide для указанной конфигурации.
     * @param int $sourceId ID источника файлов.
     * @param string|null $watermarkPath Путь к файлу водяного знака.
     * @return array|null Массив с сервером Glide и конфигурацией.
     */
    protected function getInstance($sourceId, $watermarkPath = null)
    {
        $instanceKey = $sourceId . '_' . md5($watermarkPath ? $watermarkPath : 'no_watermark');
        if (isset($this->instances[$instanceKey])) {
            return $this->instances[$instanceKey];
        }

        try {
            $sourceInfo = $this->getMediaSourceInfo($sourceId);
            $cachePath = MODX_ASSETS_PATH . 'components/thumb2x/cache/';
            $cacheUrl = MODX_ASSETS_URL . 'components/thumb2x/cache/';

            if (!$sourceInfo || empty($sourceInfo['path'])) {
                $this->modx->log(modX::LOG_LEVEL_ERROR, '[Thumb2x] Не удалось получить путь из источника файлов с ID ' . $sourceId);
                return null;
            }
            $sourcePath = $sourceInfo['path'];

            if (!is_dir($sourcePath) || !is_readable($sourcePath)) return null;
            if (!is_dir($cachePath)) @mkdir($cachePath, 0755, true);
            if (!is_writable($cachePath)) return null;

            $driver = $this->modx->getOption('thumb2x.driver', null, 'gd');

            $fileRights = octdec($this->modx->getOption('thumb2x.cache_file_rights', null, '0644'));
            $dirRights = octdec($this->modx->getOption('thumb2x.cache_dir_rights', null, '0755'));

            $visibility = \League\Flysystem\UnixVisibility\PortableVisibilityConverter::fromArray([
                'file' => ['public' => $fileRights, 'private' => 0600],
                'dir'  => ['public' => $dirRights,  'private' => 0700],
            ], \League\Flysystem\Visibility::PUBLIC);



            $sourceAdapter = new \League\Flysystem\Local\LocalFilesystemAdapter($sourcePath);
            $cacheAdapter = new \League\Flysystem\Local\LocalFilesystemAdapter($cachePath, $visibility);

            $sourceFs = new \League\Flysystem\Filesystem($sourceAdapter);
            $cacheFs = new \League\Flysystem\Filesystem($cacheAdapter);

            $glideConfig = [
                'source' => $sourceFs,
                'cache' => $cacheFs,
                'base_url' => $cacheUrl,
                'cache_with_file_extensions' => true,
                'driver' => $driver,
            ];


            $hasWatermarks = false;
            if (!empty($watermarkPath)) {
                $fullWatermarkPath = MODX_BASE_PATH . ltrim($watermarkPath, '/');
                if (is_file($fullWatermarkPath)) {
                    $watermarkDir = dirname($fullWatermarkPath);
                    $watermarkAdapter = new \League\Flysystem\Local\LocalFilesystemAdapter($watermarkDir);
                    $glideConfig['watermarks'] = new \League\Flysystem\Filesystem($watermarkAdapter);
                    $hasWatermarks = true;
                } else {
                    $this->modx->log(modX::LOG_LEVEL_ERROR, '[Thumb2x] Файл водяного знака не найден: ' . $fullWatermarkPath);
                }
            }

            $glideServer = \League\Glide\ServerFactory::create($glideConfig);

            return $this->instances[$instanceKey] = [
                'server' => $glideServer,
                'cache_path' => $cachePath,
                'has_watermarks' => $hasWatermarks,
                'source_info' => $sourceInfo,
            ];

        } catch (Exception $e) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[Thumb2x] Критическая ошибка при инициализации Glide: ' . $e->getMessage());
            return null;
        }
    }

    /**
     * Получает расширенную информацию из источника файлов.
     * @param int $sourceId
     * @return array|null
     */
    protected function getMediaSourceInfo($sourceId)
    {
        $source = $this->modx->getObject('modMediaSource', $sourceId);
        if (!$source) {
            return null;
        }
        $source->initialize();
        $properties = $source->getProperties();

        return [
            'path' => $source->getBasePath(),
            'url' => $source->getBaseUrl(),
            'image_extensions' => isset($properties['imageExtensions']['value']) ? $properties['imageExtensions']['value'] : 'jpg,jpeg,png,gif,svg,webp',
            'thumbnail_quality' => isset($properties['thumbnailQuality']['value']) ? (int)$properties['thumbnailQuality']['value'] : 90,
            'thumbnail_type' => isset($properties['thumbnailType']['value']) ? $properties['thumbnailType']['value'] : 'jpg',
        ];
    }

    /**
     * Генерирует URL для изображения с указанными параметрами.
     * @param string $path Путь к исходному файлу.
     * @param array $params Параметры для Glide.
     * @param int $sourceId ID источника файлов.
     * @param array $overrides Массив для переопределения настроек.
     * @return string
     */
    public function getUrl($path, $params = [], $sourceId = 0, $overrides = [])
    {
        // Если ID источника не передан, берем из системных настроек
        if (empty($sourceId)) {
            $sourceId = (int)$this->modx->getOption('thumb2x.source_id', null, 1);
        }

        $watermarkPath = isset($overrides['watermark_path']) ? $overrides['watermark_path'] : null;
        $instance = $this->getInstance($sourceId, $watermarkPath);
        if (!$instance) {
            return 'Ошибка: Glide не был инициализирован.';
        }

        /** @var \League\Glide\Server $glide */
        $glide = $instance['server'];
        $cachePath = $instance['cache_path'];
        $hasWatermarks = $instance['has_watermarks'];
        $sourceInfo = $instance['source_info'];

        try {
            $allowedExtsList = isset($overrides['image_extensions']) ? $overrides['image_extensions'] : $sourceInfo['image_extensions'];
            $allowedExts = explode(',', strtolower(str_replace(' ', '', $allowedExtsList)));
            $fileExt = strtolower(pathinfo($path, PATHINFO_EXTENSION));
            if (!empty($fileExt) && !in_array($fileExt, $allowedExts)) {
                return 'Недопустимый тип файла.';
            }

            $defaultParams = [
                'q' => isset($overrides['thumbnail_quality']) ? $overrides['thumbnail_quality'] : $sourceInfo['thumbnail_quality'],
                'fm' => strtolower(isset($overrides['thumbnail_type']) ? $overrides['thumbnail_type'] : $sourceInfo['thumbnail_type']),
            ];
            $finalParams = array_merge($defaultParams, $params);

            if ($watermarkPath && $hasWatermarks && !isset($finalParams['mark'])) {
                $finalParams['mark'] = basename($watermarkPath);
            }

            $hash = sha1($path . http_build_query($finalParams));
            $logEntry = $this->modx->getObject('Thumb2xFile', ['hash' => $hash]);

            if ($logEntry) {
                $logEntry->set('accessedon', date('Y-m-d H:i:s'));
                $logEntry->save();
                return $logEntry->get('cache_path');
            }

            if (!$glide->getSource()->fileExists($path)) {
                return 'Исходный файл не найден.';
            }

            $cacheFile = $glide->makeImage($path, $finalParams);
            $fullCachePath = rtrim($cachePath, '/') . '/' . ltrim($cacheFile, '/');
            $baseUrl = rtrim($glide->getBaseUrl(), '/');
            $finalUrl = $baseUrl . '/' . ltrim($cacheFile, '/');

            if (!file_exists($fullCachePath)) {
                return 'Ошибка создания файла в кэше.';
            }

            $imageSize = getimagesize($fullCachePath);
            $newLogEntry = $this->modx->newObject('Thumb2xFile');
            $newLogEntry->fromArray([
                'hash' => $hash,
                'source_path' => $path,
                'cache_path' => $finalUrl,
                'params' => json_encode($finalParams),
                'cache_filesize' => filesize($fullCachePath),
                'cache_width' => isset($imageSize[0]) ? $imageSize[0] : 0,
                'cache_height' => isset($imageSize[1]) ? $imageSize[1] : 0,
                'createdon' => date('Y-m-d H:i:s'),
                'accessedon' => date('Y-m-d H:i:s'),
            ]);
            $newLogEntry->save();

            return $finalUrl;
        } catch (Exception $e) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[Thumb2x] Ошибка генерации: ' . $e->getMessage());
            return 'Ошибка генерации.';
        }
    }
}